import {Injectable} from '@angular/core';
import {ApiService} from './api.service';
import {HttpClient, HttpErrorResponse} from '@angular/common/http';
import {DataCard, DataCardResponse} from '@nspop/gm-web-facade-api';
import {BehaviorSubject, Observable, throwError} from 'rxjs';
import {CprDoesNotExistError} from '../services/error.service';
import {tap} from 'rxjs/operators';
import {PatientContextService} from '../services/patient-context.service';

@Injectable({
  providedIn: 'root',
})
export class DataCardService extends ApiService {

  constructor(httpClient: HttpClient, private patientContextService: PatientContextService) {
    super('/data-card', httpClient);
  }

  // Inner subjects - for manually publishing updates to subscribers
  private innerDataCardResponses: BehaviorSubject<DataCardResponse[]> = new BehaviorSubject<DataCardResponse[]>([]);
  private innerActiveDataCardResponse: BehaviorSubject<DataCardResponse | undefined>
    = new BehaviorSubject<DataCardResponse | undefined>(undefined);

  // Observables (from inner subjects) - subscribe to these to get updated info!
  public allDataCards$: Observable<DataCardResponse[]> = this.innerDataCardResponses.asObservable();
  public activeDataCard$: Observable<DataCardResponse | undefined> = this.innerActiveDataCardResponse.asObservable();

  // Refresh methods - these are the ones that actually call the server and manually publishes new data
  public refresh(): void {
    this.getAll().subscribe((dataCardResponses: DataCardResponse[]) => {
      this.innerDataCardResponses.next(dataCardResponses);
      this.refreshActiveDataCard();
    });
  }

  public refreshActiveDataCard(): void {
    const dataCardResponses: DataCardResponse[] = this.innerDataCardResponses.getValue();
    const episodeOfCareId = this.patientContextService.getCurrentEpisodeOfCare();
    this.innerActiveDataCardResponse.next(dataCardResponses.find(card => card.episodeOfCareIdentifier === episodeOfCareId));
  }

  // Overriding api.service methods
  public create(): Observable<DataCardResponse> {
    return super.create(null);
  }

  public update(resource: DataCard, documentUniqueId: string, version: string): Observable<DataCardResponse> {
    return super.update(resource, documentUniqueId, version).pipe(
      tap((dataCardResponse: DataCardResponse) => {
        this.innerActiveDataCardResponse.next(dataCardResponse);
      })
    );
  }

  protected handleGetAllError = (error: HttpErrorResponse): Observable<never> => {
    if (error.status === 400) {
      return throwError(new CprDoesNotExistError(error));
    }

    if (error.status === 404) {
      return throwError(new CprDoesNotExistError(error));
    }
    return super.handleGetAllError(error);
  }
}
