import {Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges, ViewChild} from '@angular/core';
import {ModalComponent} from '../../modals/modal/modal.component';
import {
  CreateRegularObservationsRequest,
  CreateUltrasoundObservationsRequest,
  DataCardResponse
} from '@nspop/gm-web-facade-api';
import {FormComponent} from '../../confirm-navigation-if-dirty.framework';
import {NgForm} from '@angular/forms';
import {copy} from '../../../utils/copy';
import {merge, Observable, of} from 'rxjs';
import {finalize} from 'rxjs/operators';
import {DataCardWrapper} from '../../model-wrappers/dataCard.wrapper';
import {ObservationService} from '../../api/observation.service';
import {DataCardService} from '../../api/data-card.service';


type ObservationFormType = 'regular' | 'ultrasound';
@Component({
  selector: 'app-edit-measurements-modal',
  templateUrl: './edit-measurements-modal.component.html',
  styleUrls: ['./edit-measurements-modal.component.scss']
})
export class EditMeasurementsModalComponent extends FormComponent implements OnChanges, OnInit {
  public note = '';
  public isSaving = false;
  public dataCard?: DataCardWrapper;
  public observationData?: CreateUltrasoundObservationsRequest|CreateRegularObservationsRequest | undefined = undefined;

  @ViewChild('modal') public modalComponent: ModalComponent | undefined;
  @ViewChild('form') public form: NgForm | undefined;
  @Input() public isOpen = false;
  @Input() public observationType: ObservationFormType = 'regular';
  @Input() public allowTypeChange = false;
  @Output() public modalClosed: EventEmitter<void> = new EventEmitter();
  @Output() public save: EventEmitter<void> = new EventEmitter();
  private identifier = '';
  private version = '';

  constructor(
    private dataCardService: DataCardService,
    private observationService: ObservationService,
  ) {
    super();
  }

  ngOnInit(): void {
    this.dataCardService.activeDataCard$.subscribe((card?: DataCardResponse) => {
      if (card) {
        const {documentUniqueId, version, episodeOfCareIdentifier, ...dataCard} = card;
        this.identifier = documentUniqueId || '';
        this.version = documentUniqueId || '';
        this.dataCard = new DataCardWrapper(copy(dataCard));
        this.changeObservationType(this.observationType);
      }
    });
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.isOpen) {
      if (this.isOpen) {
        this.modalComponent?.setVisible(true);
      } else {
        this.handleClose();
      }
    }
  }

  public handleClose(): void {
    if (this.observationData) {
      this.observationData.activityId = undefined;
    }
    this.modalComponent?.setVisible(false);
    this.modalClosed.emit();
  }

  public validateActivityId(): boolean {
    const activitySelector = document.getElementById('activitySelector');
    if (!this.observationData?.activityId && this.modalComponent?.isModalVisible) {
      activitySelector?.classList.add('ng-invalid');
      return false;
    }
    activitySelector?.classList.remove('ng-invalid');
    return true;
  }

  public changeObservationType(formType: ObservationFormType): void {
    this.observationData =  {
      activityId: this.observationData?.activityId,
      fetuses: [...new Array(this.dataCard?.fetusCount || 1)].map((x, index) => ({id: `Foster ${index + 1}`}))
    };
    if (formType === 'regular') {
      (this.observationData as CreateRegularObservationsRequest).bloodPressure = undefined;
    }
    if (formType === 'ultrasound') {
      delete (this.observationData as CreateRegularObservationsRequest).bloodPressure;
    }
  }

  public doSave(): void {
    if (!this.validateActivityId()) {
      const activitySelector = document.getElementById('activitySelector');
      activitySelector?.scrollIntoView({behavior: 'smooth', block: 'center'});
      return;
    }
    this.isSaving = true;
    const save = (this.observationType === 'ultrasound') ? this.saveUltrasound() : this.saveRegular();
    save// TODO: create note too (Eller hvad? Skal vi se om der allerede er en på aktiviteten og hvis ja redigere den?)
      .pipe(finalize(() => this.isSaving = false))
    .subscribe(() => {
      this.handleClose();
      this.dataCardService.refresh();
      this.observationService.refresh();
      this.save.emit();
    });
  }

  private saveUltrasound(): Observable<void> {
    if (this.dataCard) {
      return merge(
        this.dataCardService.update(this.dataCard.asDataCard, this.identifier, this.version),
        this.observationService.createUltrasoundObservations(this.observationData as CreateUltrasoundObservationsRequest)
      ) as unknown as Observable<void>;
    }
    return of();
  }

  private saveRegular(): Observable<void> {
    return this.observationService.createRegularObservations(this.observationData as CreateRegularObservationsRequest);
  }
}
